@@ -1,5 +1,6 @@
 package Chemistry::Mol;
-$VERSION = '0.11';
+$VERSION = '0.20';
+# $Id: Mol.pm,v 1.24 2004/04/15 17:37:18 ivan Exp $
 
 =head1 NAME
 
@@ -36,7 +37,15 @@ use strict;
 use Chemistry::Atom;
 use Chemistry::Bond;
 use Carp;
-use base qw(Chemistry::Obj);
+use base qw(Chemistry::Obj Exporter);
+use Storable 'dclone';
+
+our @EXPORT_OK = qw(read_mol);
+our @EXPORT = ();
+our %EXPORT_TAGS = (
+      all  => [@EXPORT, @EXPORT_OK],
+);
+
 
 
 my %FILE_FORMATS = ();
@@ -92,9 +101,13 @@ Add one or more Atom objects to the molecule. Returns the last atom added.
 
 sub add_atom {
     my $self = shift;
-    for my $a (@_){
-        push @{$self->{atoms}}, $a;
-        $self->{byId}{$a->{id}} = $a;
+    for my $atom (@_){
+        #if ($self->by_id($atom->id)) {
+            #croak "Duplicate ID when adding atom '$atom' to mol '$self'";
+        #}
+        push @{$self->{atoms}}, $atom;
+        $self->{byId}{$atom->id} = $atom;
+        $atom->parent($self);
     }
     $_[-1];
 }
@@ -112,6 +125,35 @@ sub new_atom {
     $self->add_atom(Chemistry::Atom->new(@_));
 }
 
+=item $mol->delete_atom($atom, ...)
+
+Deletes an atom from the molecule. It automatically deletes all the bonds
+in which the atom participates as well.
+
+=cut
+
+# mol deletes bonds that belonged to atom
+# mol deletes atom
+
+sub delete_atom {
+    my $self = shift;
+    for my $i (@_){
+        my ($atom, $index);
+        if (ref $i) {
+            $atom = $i;
+            $index = $self->get_atom_index($atom)    
+                or croak "$self->delete_atom: no such atom $atom\n";
+        } else {
+            $index = $i;
+            $atom = $self->atoms($index)
+                or croak "$self->delete_atom: no such atom $index\n";
+        }
+        my $id = $atom->id;
+        $self->delete_bond($atom->bonds);
+        delete $self->{byId}{$id};
+        splice @{$self->{atoms}}, $index - 1, 1;
+    }
+}
 
 =item $mol->add_bond($bond, ...)
 
@@ -121,9 +163,13 @@ Add one or more Bond objects to the molecule. Returns the last bond added.
 
 sub add_bond {
     my $self = shift;
-    for my $b (@_){
-        push @{$self->{bonds}}, $b;
-	$self->{byId}{$b->{id}} = $b;
+    for my $bond (@_){
+        #if ($self->by_id($bond->id)) {
+            #croak "Duplicate ID when adding bond '$bond' to mol '$self'";
+        #}
+        push @{$self->{bonds}}, $bond;
+	$self->{byId}{$bond->id} = $bond;
+        $bond->parent($self);
     }
     $_[-1];
 }
@@ -141,6 +187,55 @@ sub new_bond {
     $self->add_bond(Chemistry::Bond->new(@_));
 }
 
+sub get_bond_index {
+    my ($self, $bond) = @_;
+    my $i;
+    for ($self->bonds) {
+        ++$i;
+        return $i if ($_ eq $bond);
+    }
+    undef;
+}
+
+sub get_atom_index {
+    my ($self, $atom) = @_;
+    my $i;
+    for ($self->atoms) {
+        ++$i;
+        return $i if ($_ eq $atom);
+    }
+    undef;
+}
+
+=item $mol->delete_bond($bond, ...)
+
+Deletes a bond from the molecule.
+
+=cut
+
+# mol deletes bond
+# bond tells atoms involved to forget about it
+
+sub delete_bond {
+    my $self = shift;
+    for my $i (@_){
+        my ($bond, $index);
+        if (ref $i) {
+            $bond = $i;
+            $index = $self->get_bond_index($bond)
+                or croak "$self->delete_bond: no such bond $bond\n";
+        } else {
+            $index = $i;
+            $bond = $self->bonds($index)
+                or croak "$self->delete_bond: no such bond $index\n";
+        }
+        my $id = $bond->id;
+        delete $self->{byId}{$id};
+        splice @{$self->{bonds}}, $index - 1, 1;
+        $bond->delete_atoms;
+    }
+}
+
 =item $mol->by_id($id)
 
 Return the atom or bond object with the corresponding id.
@@ -270,7 +365,7 @@ be registered using register_type(); modules that include readers
 
 sub read_mol { # for backwards compatibility
     my ($fname, $type) = shift;
-    Chemistry::Mol->read($fname, format => $type);
+    __PACKAGE__->read($fname, format => $type);
 }
 
 sub read {
@@ -312,7 +407,7 @@ sub write {
 	    }
 	}
     }
-    croak "Couldn't guess format of file '$fname'";
+    croak "Couldn't guess format for writing file '$fname'";
 }
 
 =item Chemistry::Mol->register_format($name, $ref)
@@ -406,28 +501,151 @@ sub formula {
     $self->print(format => "formula", formula_format => $format);
 }
 
+=item my $mol2 = $mol->clone;
+
+Makes a copy of a molecule.
+
+=cut
+
+sub clone {
+    my ($self) = @_;
+    my $clone = dclone $self;
+    for ($clone->atoms, $clone->bonds) {
+        $_->_weaken;
+    }
+    $clone;
+}
+
+=item ($distance, $atom_here, $atom_there) = $mol->distance($obj)
+
+Returns the minimum distance to $obj, which can be an atom, a molecule, or a
+vector. In scalar context it returns only the distance; in list context it
+also returns the atoms involved. The current implementation for calculating
+the minimum distance between two molecules compares every possible pair of
+atoms, so it's not efficient for large molecules.
+
+=cut
+
+sub distance {
+    my ($self, $other) = @_;
+    if ($other->isa("Chemistry::Mol")) {
+        my @atoms = $self->atoms;
+        my $atom = shift @atoms or return undef; # need at least one atom
+        my $closest_here = $atom;
+        my ($min_length, $closest_there) = $atom->distance($other);
+        for $atom (@atoms) {
+            my ($d, $o) = $atom->distance($other);
+            if ($d < $min_length) {
+                ($min_length, $closest_there, $closest_here) = ($d, $o, $atom);
+            }
+        }
+        return wantarray ? 
+            ($min_length, $closest_here, $closest_there) : $min_length;
+    } elsif ($other->isa("Chemistry::Atom")) {
+        return $other->distance($self);
+    } elsif ($other->isa("Math::VectorReal")) {
+        return Chemistry::Atom->new(coords => $other)->distance($self);
+    }
+}
+
+=item my $bigmol = Chemistry::Mol->combine($mol1, $mol2, ...)
+
+=item $mol1->combine($mol2, $mol3, ...)
+
+Combines several molecules in one bigger molecule. If called as a class method,
+as in the first example, it returns a new combined molecule without altering
+any of the parameters. If called as an instance method, as in the second
+example, all molecules are combined into $mol1 (but $mol2, $mol3, ...) are not
+altered.
+
+=cut
+
+# joins several molecules into one
+# Does not touch the original copy.
+sub combine {
+    my ($self, @others) = @_;
+    my $mol;
+    if (ref $self) {
+        $mol = $self;
+    } else {
+        $mol = $self->new;
+    }
+    for my $other (@others) {
+        my $mol2 = $other->clone;
+        for my $atom ($mol2->atoms) {
+            $mol->add_atom($atom);
+        }
+        for my $bond ($mol2->bonds) {
+            $mol->add_bond($bond);
+        }
+    }
+    $mol;
+}
+
+=item my @mols = $mol->separate
+
+Separates a molecule into "connected fragments". The original object is not
+modified; the fragments are clones of the original ones. Example: if you have
+ethane (H3CCH3) and you delete the C-C bond, you have two CH3 radicals within
+one molecule object ($mol). When you call $mol->separate you get two molecules,
+each one with a CH3.
+
+=cut
+
+# splits a molecule into connected fragments
+# returns a list of molecules. Does not touch the original copy.
+sub separate {
+    my ($self) = @_;
+    $self = $self->clone;
+    $self->{_paint_tab} = {};
+    my $color = 0;
+    for my $atom ($self->atoms) {
+        next if defined $self->{_paint_tab}{$atom->id};
+        $self->_paint($atom, $color++);
+    }
+    my @mols;
+    push @mols, $self->new for (1 .. $color);
+    for my $atom ($self->atoms) {
+        $mols[$self->{_paint_tab}{$atom->id}]->add_atom($atom);
+    }
+    for my $bond ($self->bonds) {
+        $mols[$self->{_paint_tab}{$bond->id}]->add_bond($bond);
+    }
+    @mols;
+}
+
+sub _paint {
+    my ($self, $atom, $color) = @_;
+    return if $self->{_paint_tab}{$atom->id} eq $color;
+    $self->{_paint_tab}{$atom->id} = $color;
+    $self->{_paint_tab}{$_->id} = $color for ($atom->bonds);
+    for my $neighbor ($atom->neighbors) {
+        $self->_paint($neighbor, $color);
+    }
+}
+
 1;
 
 =back
 
-=head1 BUGS
+=head1 VERSION
 
-Blatant memory leaks. Due to the use of circular references, Perl's current
-garbage collector never cleans up molecule, atom, and bond objects. A future
-version should address this.
+0.20
 
 =head1 SEE ALSO
 
 L<Chemistry::Atom>, L<Chemistry::Bond>, L<Chemistry::File>,
 L<Chemistry::Tutorial>
 
+The PerlMol website L<http://www.perlmol.org/>
+
 =head1 AUTHOR
 
 Ivan Tubert E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 Ivan Tubert. All rights reserved. This program is free
+Copyright (c) 2004 Ivan Tubert. All rights reserved. This program is free
 software; you can redistribute it and/or modify it under the same terms as
 Perl itself.
 
