@@ -1,5 +1,6 @@
 package Chemistry::Obj;
-$VERSION = "0.11";
+$VERSION = "0.20";
+# $Id: Obj.pm,v 1.12 2004/04/15 17:37:18 ivan Exp $
 use 5.006001;
 
 use strict;
@@ -161,11 +162,24 @@ sub obj_cmp {
 
 accessor(qw(id name type));
 
+sub use {
+    my ($pack, $module, @args) = @_;
+    $pack = ref $pack || $pack;
+    my $args = @args ? "(@args)" : '';
+    eval "package $pack; use $module $args";
+}
+
 1;
 
+=head1 VERSION
+
+0.20
+
 =head1 SEE ALSO
 
-Chemistry::Atom, Chemistry::Bond, Chemistry::Mol
+L<Chemistry::Atom>, L<Chemistry::Bond>, L<Chemistry::Mol>
+
+The PerlMol website L<http://www.perlmol.org/>
 
 =head1 AUTHOR
 
@@ -173,7 +187,7 @@ Ivan Tubert E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 Ivan Tubert. All rights reserved. This program is free
+Copyright (c) 2004 Ivan Tubert. All rights reserved. This program is free
 software; you can redistribute it and/or modify it under the same terms as
 Perl itself.
 
